@@ -4,16 +4,16 @@
  * Copyright (C) 2008, Nokia
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
@@ -140,17 +140,25 @@ tracker_log_handler (const gchar    *domain,
 }
 
 gboolean
-tracker_log_init (const gchar *filename,
-		  gint	       this_verbosity)
+tracker_log_init (gint    this_verbosity,
+		  gchar **used_filename)
 {
-	g_return_val_if_fail (filename != NULL, FALSE);
+	gchar *filename;
+	gchar *basename;
 
 	if (initialized) {
 		return TRUE;
 	}
+		
+	basename = g_strdup_printf ("%s.log", g_get_application_name ());
+	filename = g_build_filename (g_get_user_data_dir (), 
+				     "tracker", 
+				     basename, 
+				     NULL);
+	g_free (basename);
 
 	/* Remove previous log */
-	tracker_file_unlink (filename);
+	g_unlink (filename);
 
 	/* Open file */
 	fd = g_fopen (filename, "a");
@@ -177,6 +185,12 @@ tracker_log_init (const gchar *filename,
 
 	g_log_set_default_handler (tracker_log_handler, NULL);
 
+	if (used_filename) {
+		*used_filename = filename;
+	} else {
+		g_free (filename);
+	}
+
 	initialized = TRUE;
 
 	return TRUE;
