@@ -27,14 +27,23 @@
 #include <gio/gio.h>
 
 #include <libtracker-common/tracker-dbus.h>
+#include <libtracker-common/tracker-sparql-builder.h>
 
-#include "tracker-main.h"
 #include "tracker-dbus.h"
 #include "tracker-extract.h"
+#include "tracker-main.h"
+#include "tracker-marshal.h"
+
+#ifdef HAVE_STREAMANALYZER
+#include "tracker-topanalyzer.h"
+#endif
+
+#define MAX_EXTRACT_TIME 10
 
-#define MAX_EXTRACT_TIME 5
 #define TRACKER_EXTRACT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_EXTRACT, TrackerExtractPrivate))
 
+extern gboolean debug;
+
 typedef struct {
 	GArray *specific_extractors;
 	GArray *generic_extractors;
@@ -45,8 +54,15 @@ typedef struct {
 	const TrackerExtractData *edata; 
 }  ModuleData;
 
+enum {
+	QUEUE_THUMBNAIL,
+	LAST_SIGNAL
+};
+
 static void tracker_extract_finalize (GObject *object);
 
+static guint signals[LAST_SIGNAL] = {0};
+
 G_DEFINE_TYPE(TrackerExtract, tracker_extract, G_TYPE_OBJECT)
 
 static void
@@ -58,12 +74,27 @@ tracker_extract_class_init (TrackerExtractClass *klass)
 
 	object_class->finalize = tracker_extract_finalize;
 
+	signals[QUEUE_THUMBNAIL] =
+		g_signal_new ("queue-thumbnail",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      tracker_marshal_VOID__STRING_STRING,
+			      G_TYPE_NONE,
+			      2,
+			      G_TYPE_STRING,
+			      G_TYPE_STRING);
+
 	g_type_class_add_private (object_class, sizeof (TrackerExtractPrivate));
 }
 
 static void
 tracker_extract_init (TrackerExtract *object)
 {
+#ifdef HAVE_STREAMANALYZER
+	tracker_topanalyzer_init ();
+#endif
 }
 
 static void
@@ -72,7 +103,11 @@ tracker_extract_finalize (GObject *object)
 	TrackerExtractPrivate *priv;
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
-	
+
+#ifdef HAVE_STREAMANALYZER
+	tracker_topanalyzer_shutdown ();
+#endif
+
 	g_array_free (priv->specific_extractors, TRUE);
 	g_array_free (priv->generic_extractors, TRUE);
 
@@ -173,138 +208,79 @@ tracker_extract_new (void)
 	return object;
 }
 
-static void
-print_file_metadata_item (gpointer key,
-			  gpointer value,
-			  gpointer user_data)
-{
-	gchar *value_utf8;
-
-	g_return_if_fail (key != NULL);
-	g_return_if_fail (value != NULL);
-
-	value_utf8 = g_locale_to_utf8 (value, -1, NULL, NULL, NULL);
-
-	if (value_utf8) {
-		tracker_dbus_request_debug (GPOINTER_TO_UINT (user_data),
-					    "  Found '%s'='%s'",
-					    key,
-					    value_utf8);
-		g_free (value_utf8);
-	}
-}
-
-static GHashTable *
+static TrackerSparqlBuilder *
 get_file_metadata (TrackerExtract *extract,
 		   guint           request_id,
-		   const gchar    *path,
+		   const gchar    *uri,
 		   const gchar    *mime)
 {
-	GHashTable *values;
-	GFile *file;
-	GFileInfo *info;
-	GError *error = NULL;
-	const gchar *attributes = NULL;
-	gchar *path_in_locale;
-	gchar *path_used;
+	TrackerSparqlBuilder *statements;
 	gchar *mime_used = NULL;
-	goffset size = 0;
-		
-	path_used = g_strdup (path);
-	g_strstrip (path_used);
+	gchar *content_type = NULL;
 
-	path_in_locale = g_filename_from_utf8 (path_used, -1, NULL, NULL, NULL);
-	g_free (path_used);
-
-	if (!path_in_locale) {
-		g_warning ("Could not convert path from UTF-8 to locale");
-		g_free (path_used);
-		return NULL;
-	}
-
-	file = g_file_new_for_path (path_in_locale);
-	if (!file) {
-		g_warning ("Could not create GFile for path:'%s'",
-			   path_in_locale);
-		g_free (path_in_locale);
-		return NULL;
-	}
-		
-	/* Blocks */
-	if (!g_file_query_exists (file, NULL)) {
-		g_warning ("File does not exist '%s'", path_in_locale);
-		g_object_unref (file);
-		g_free (path_in_locale);
-		return NULL;
-	}
-
-	/* Do we get size and mime? or just size? */
-	if (mime && *mime) {	
-		attributes = 
-			G_FILE_ATTRIBUTE_STANDARD_SIZE;
-	} else {
-		attributes = 
-			G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-			G_FILE_ATTRIBUTE_STANDARD_SIZE;
-	}
-
-	info = g_file_query_info (file, 
-				  attributes, 
-				  G_FILE_QUERY_INFO_NONE, 
-				  NULL, 
-				  &error);
-	
-	if (error || !info) {
-		tracker_dbus_request_comment (request_id,
-					      "  Could not create GFileInfo for file size check, %s",
-					      error ? error->message : "no error given");
-		g_error_free (error);
-		
-		if (info) {
-			g_object_unref (info);
-		}	
-		
-		g_object_unref (file);
-		g_free (path_in_locale);
-		
-		return NULL;
-	}
-	
 	/* Create hash table to send back */
-	values = g_hash_table_new_full (g_str_hash,
-					g_str_equal,
-					g_free,
-					g_free);
+	statements = tracker_sparql_builder_new_update ();
 
-	/* Check the size is actually non-zero */
-	size = g_file_info_get_size (info);
+	tracker_sparql_builder_insert_open (statements);
 
-	if (size < 1) {
-		tracker_dbus_request_comment (request_id,
-					      "  File size is 0 bytes, ignoring file");
-		
-		g_object_unref (info);
-		g_object_unref (file);
-		g_free (path_in_locale);
+#ifdef HAVE_STREAMANALYZER
+	tracker_topanalyzer_extract (uri, statements, &content_type);
 
-		return values;
+	if ((!mime || mime[0]=='\0') && content_type) {
+		mime = content_type;
 	}
+#endif
 
-	/* We know the mime */
 	if (mime && *mime) {
+		/* We know the mime */
 		mime_used = g_strdup (mime);
 		g_strstrip (mime_used);
 	} else {
+		GFile *file;
+		GFileInfo *info;
+		GError *error = NULL;
+
+		file = g_file_new_for_uri (uri);
+		if (!file) {
+			g_warning ("Could not create GFile for uri:'%s'",
+				   uri);
+			g_free (content_type);
+			g_object_unref (statements);
+			return NULL;
+		}
+
+		info = g_file_query_info (file,
+					  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					  G_FILE_QUERY_INFO_NONE, 
+					  NULL, 
+					  &error);
+	
+		if (error || !info) {
+			tracker_dbus_request_comment (request_id,
+						      "  Could not create GFileInfo for file size check, %s",
+						      error ? error->message : "no error given");
+			g_error_free (error);
+		
+			if (info) {
+				g_object_unref (info);
+			}
+
+			g_object_unref (file);
+			g_free (content_type);
+			g_object_unref (statements);
+			return NULL;
+		}
+
 		mime_used = g_strdup (g_file_info_get_content_type (info));
 
 		tracker_dbus_request_comment (request_id,
-					      "  Guessing mime type as '%s' for path:'%s'",
+					      "  Guessing mime type as '%s' for uri:'%s'",
 					      mime_used,
-					      path_in_locale);
-	}
+					      uri);
 
-	g_object_unref (info);
-	g_object_unref (file);
+		g_object_unref (info);
+		g_object_unref (file);
+	}
 
 	/* Now we have sanity checked everything, actually get the
 	 * data we need from the extractors.
@@ -329,9 +305,9 @@ get_file_metadata (TrackerExtract *extract,
 							      "  Extracting with module:'%s'",
 							      g_module_name ((GModule*) mdata.module));
 
-				(*edata->extract) (path_in_locale, values);
+				(*edata->extract) (uri, statements);
 
-				items = g_hash_table_size (values);
+				items = tracker_sparql_builder_get_length (statements);
 
 				tracker_dbus_request_comment (request_id,
 							      "  Found %d metadata items",
@@ -340,10 +316,12 @@ get_file_metadata (TrackerExtract *extract,
 					continue;
 				}
 
-				g_free (path_in_locale);
-				g_free (mime_used);
+				tracker_sparql_builder_insert_close (statements);
 
-				return values;
+				g_free (mime_used);
+				g_free (content_type);
+
+				return statements;
 			}
 		}
 
@@ -361,9 +339,9 @@ get_file_metadata (TrackerExtract *extract,
 							      "  Extracting with module:'%s'",
 							      g_module_name ((GModule*) mdata.module));
 				
-				(*edata->extract) (path_in_locale, values);
+				(*edata->extract) (uri, statements);
 
-				items = g_hash_table_size (values);
+				items = tracker_sparql_builder_get_length (statements);
 
 				tracker_dbus_request_comment (request_id,
 							      "  Found %d metadata items",
@@ -372,10 +350,12 @@ get_file_metadata (TrackerExtract *extract,
 					continue;
 				}
 
-				g_free (path_in_locale);
-				g_free (mime_used);
+				tracker_sparql_builder_insert_close (statements);
 
-				return values;
+				g_free (mime_used);
+				g_free (content_type);
+
+				return statements;
 			}
 		}
 
@@ -388,37 +368,38 @@ get_file_metadata (TrackerExtract *extract,
 					      "  No mime available, not extracting data");
 	}
 
-	g_free (path_in_locale);
+	tracker_sparql_builder_insert_close (statements);
 
-	return values;
+	g_free (content_type);
+
+	return statements;
 }
 
 void
 tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
-					 const gchar    *path,
+					 const gchar    *uri,
 					 const gchar    *mime)
 {
 	guint       request_id;
-	GHashTable *values = NULL;
+	TrackerSparqlBuilder *statements = NULL;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
-	g_return_if_fail (path != NULL);
+	g_return_if_fail (uri != NULL);
 
 	tracker_dbus_request_new (request_id,
 				  "Command line request to extract metadata, "
-				  "path:'%s', mime:%s",
-				  path,
+				  "uri:'%s', mime:%s",
+				  uri,
 				  mime);
 
 	/* NOTE: Don't reset the timeout to shutdown here */
-	values = get_file_metadata (object, request_id, path, mime);
+	statements = get_file_metadata (object, request_id, uri, mime);
 
-	if (values) {
-		g_hash_table_foreach (values, 
-				      print_file_metadata_item, 
-				      GUINT_TO_POINTER (request_id));
-		g_hash_table_destroy (values);
+	if (statements) {
+		tracker_dbus_request_debug (request_id, "%s",
+		                            tracker_sparql_builder_get_result (statements));
+		g_object_unref (statements);
 	}
 
 	tracker_dbus_request_success (request_id);
@@ -435,7 +416,7 @@ tracker_extract_get_pid (TrackerExtract	        *object,
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
-				  "DBus request to get PID");
+				  "D-Bus request to get PID");
 	
 	value = getpid ();
 	tracker_dbus_request_debug (request_id,
@@ -448,51 +429,56 @@ tracker_extract_get_pid (TrackerExtract	        *object,
 
 void
 tracker_extract_get_metadata (TrackerExtract	     *object,
-			      const gchar            *path,
+			      const gchar            *uri,
 			      const gchar            *mime,
 			      DBusGMethodInvocation  *context,
 			      GError		    **error)
 {
 	guint       request_id;
-	GHashTable *values = NULL;
+	TrackerSparqlBuilder *sparql = NULL;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
-	tracker_dbus_async_return_if_fail (path != NULL, context);
+	tracker_dbus_async_return_if_fail (uri != NULL, context);
 
 	tracker_dbus_request_new (request_id,
-				  "DBus request to extract metadata, "
-				  "path:'%s', mime:%s",
-				  path,
+				  "D-Bus request to extract metadata, "
+				  "uri:'%s', mime:%s",
+				  uri,
 				  mime);
 
 	tracker_dbus_request_debug (request_id,
 				    "  Resetting shutdown timeout");
 	
 	tracker_main_quit_timeout_reset ();
-	alarm (MAX_EXTRACT_TIME);
+	if (!debug) {
+		alarm (MAX_EXTRACT_TIME);
+	}
 
-	values = get_file_metadata (object, request_id, path, mime);
+	sparql = get_file_metadata (object, request_id, uri, mime);
 
-	if (values) {
-		g_hash_table_foreach (values, 
-				      print_file_metadata_item, 
-				      GUINT_TO_POINTER (request_id));
-		dbus_g_method_return (context, values);
-		g_hash_table_destroy (values);
+	if (sparql) {
+		if (tracker_sparql_builder_get_length (sparql) > 0) {
+			dbus_g_method_return (context, tracker_sparql_builder_get_result (sparql));
+		} else {
+			dbus_g_method_return (context, "");
+		}
+		g_object_unref (sparql);
 		tracker_dbus_request_success (request_id);
 	} else {
 		GError *actual_error = NULL;
 
 		tracker_dbus_request_failed (request_id,
 					     &actual_error,
-					     "Could not get any metadata for path:'%s' and mime:'%s'",
-					     path, 
+					     "Could not get any metadata for uri:'%s' and mime:'%s'",
+					     uri, 
 					     mime);
 		dbus_g_method_return_error (context, actual_error);
 		g_error_free (actual_error);
 	}
 
-	/* Unset alarm so the extractor doesn't die when it's idle */
-	alarm (0);
+	if (!debug) {
+		/* Unset alarm so the extractor doesn't die when it's idle */
+		alarm (0);
+	}
 }
