@@ -29,7 +29,9 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
+#if defined(__linux__)
 #include <linux/sched.h>
+#endif
 #include <sched.h>
 
 #include <glib.h>
@@ -42,12 +44,12 @@
 #endif
 
 #include <libtracker-common/tracker-log.h>
-#include <libtracker-common/tracker-config.h>
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-os-dependant.h>
-#include <libtracker-common/tracker-thumbnailer.h>
 #include <libtracker-common/tracker-ioprio.h>
 
+#include "tracker-albumart.h"
+#include "tracker-config.h"
 #include "tracker-main.h"
 #include "tracker-dbus.h"
 #include "tracker-extract.h"
@@ -64,13 +66,14 @@
 
 #define QUIT_TIMEOUT 30 /* 1/2 minutes worth of seconds */
 
-static GMainLoop *main_loop;
-static guint      quit_timeout_id;
+static GMainLoop  *main_loop;
+static guint       quit_timeout_id = 0;
 
-static gboolean   version;
-static gint       verbosity = -1;
-static gchar     *filename;
-static gchar     *mime_type;
+static gboolean    version;
+gboolean           debug = FALSE;
+static gint        verbosity = -1;
+static gchar      *filename;
+static gchar      *mime_type;
 
 static GOptionEntry  entries[] = {
 	{ "version", 'V', 0,
@@ -86,10 +89,16 @@ static GOptionEntry  entries[] = {
 	  G_OPTION_ARG_FILENAME, &filename,
 	  N_("File to extract metadata for"),
 	  N_("FILE") },
-	{ "file", 'm', 0,
+	{ "mime", 'm', 0,
 	  G_OPTION_ARG_STRING, &mime_type,
 	  N_("MIME type for file (if not provided, this will be guessed)"),
 	  N_("MIME") },
+	/* Debug run is used to avoid that the mainloop exits, so that
+	 * as a developer you can be relax when running the tool in gdb */
+	{ "debug", 'd', 0,
+	  G_OPTION_ARG_NONE, &debug,
+	  N_("Debug (default = off)"),
+	  NULL },
 
 	{ NULL }
 };
@@ -98,7 +107,12 @@ static gboolean
 quit_timeout_cb (gpointer user_data)
 {
 	quit_timeout_id = 0;
-	g_main_loop_quit (main_loop);
+	
+	if (!debug) {
+		g_main_loop_quit (main_loop);
+	} else {
+		g_debug ("Would have quit the mainloop");
+	}
 
 	return FALSE;
 }
@@ -115,7 +129,6 @@ tracker_main_quit_timeout_reset (void)
 						 NULL);
 }
 
-
 static void
 initialize_priority (void)
 {
@@ -169,8 +182,9 @@ signal_handler (int signo)
 	}
 
 	switch (signo) {
+	case SIGABRT:
 	case SIGALRM:
-		exit (EXIT_FAILURE);
+		_exit (EXIT_FAILURE);
 		break;
 	case SIGTERM:
 	case SIGINT:
@@ -203,16 +217,63 @@ initialize_signal_handler (void)
 	sigaction (SIGINT,  &act, NULL);
 	sigaction (SIGHUP,  &act, NULL);
 	sigaction (SIGALRM,  &act, NULL);
+	sigaction (SIGABRT, &act, NULL);
 #endif /* G_OS_WIN32 */
 }
 
+static void
+log_handler (const gchar    *domain,
+	     GLogLevelFlags  log_level,
+	     const gchar    *message,
+	     gpointer	     user_data)
+{
+	if (((log_level & G_LOG_LEVEL_DEBUG) && verbosity < 3) ||
+	    ((log_level & G_LOG_LEVEL_INFO) && verbosity < 2) ||
+	    ((log_level & G_LOG_LEVEL_MESSAGE) && verbosity < 1)) {
+		return;
+	}
+
+	switch (log_level) {
+	case G_LOG_LEVEL_WARNING:
+	case G_LOG_LEVEL_CRITICAL:
+	case G_LOG_LEVEL_ERROR:
+	case G_LOG_FLAG_RECURSION:
+	case G_LOG_FLAG_FATAL:
+		g_fprintf (stderr, "%s\n", message);
+		fflush (stderr);
+		break;
+	case G_LOG_LEVEL_MESSAGE:
+	case G_LOG_LEVEL_INFO:
+	case G_LOG_LEVEL_DEBUG:
+	case G_LOG_LEVEL_MASK:
+	default:
+		g_fprintf (stdout, "%s\n", message);
+		fflush (stdout);
+		break;
+	}	
+}
+
 int
 main (int argc, char *argv[])
 {
 	GOptionContext *context;
 	GError         *error = NULL;
 	TrackerConfig  *config;
-	gchar          *log_filename;
+	gchar          *log_filename = NULL;
+	gboolean        stand_alone = FALSE;
+	guint           log_handler_id = 0;
+
+	g_type_init ();
+
+	if (!g_thread_supported ()) {
+		g_thread_init (NULL);
+	}
+
+	dbus_g_thread_init ();
+
+	g_set_application_name ("tracker-extract");
+
+	setlocale (LC_ALL, "");
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -249,90 +310,101 @@ main (int argc, char *argv[])
 		return EXIT_SUCCESS;
 	}
 
-	g_print ("Initializing tracker-extract...\n");
+	g_print ("Initializing tracker-extract...%s\n", debug ? "Running in debug mode" : "");
 
 	initialize_signal_handler ();
 
 	tracker_memory_setrlimits ();
 
-	g_type_init ();
+	/* Set conditions when we use stand alone settings */
+	stand_alone |= filename != NULL;
 
-	if (!g_thread_supported ()) {
-		g_thread_init (NULL);
+	if (stand_alone) {
+		/* Set log handler for library messages */
+		log_handler_id = g_log_set_handler (NULL,
+						    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL,
+						    log_handler,
+						    NULL);
+		
+		g_log_set_default_handler (log_handler, NULL);
+
+		/* Set the default verbosity if unset */
+		if (verbosity == -1) {
+			verbosity = 3;
+		}
 	}
 
-	dbus_g_thread_init ();
-
-	g_set_application_name ("tracker-extract");
-
-	setlocale (LC_ALL, "");
-
 	if (filename) {
-		TrackerExtract *object;
+		TrackerExtract *extract;
 		GFile *file;
-		gchar *full_path;
+		gchar *uri;
 
-		object = tracker_extract_new ();
-		if (!object) {
+		extract = tracker_extract_new ();
+		if (!extract) {
 			return EXIT_FAILURE;
 		}
 
 		file = g_file_new_for_commandline_arg (filename);
-		full_path = g_file_get_path (file);
+		uri = g_file_get_uri (file);
 
-		tracker_extract_get_metadata_by_cmdline (object, full_path, mime_type);
+		tracker_extract_get_metadata_by_cmdline (extract, uri, mime_type);
 
-		g_object_unref (object);
+		g_object_unref (extract);
 		g_object_unref (file);
-		g_free (full_path);
+		g_free (uri);
 
-		return EXIT_SUCCESS;
+		goto done;
 	}
 
 	config = tracker_config_new ();
 
-	log_filename =
-		g_build_filename (g_get_user_data_dir (),
-				  "tracker",
-				  "tracker-extract.log",
-				  NULL);
-
 	/* Extractor command line arguments */
 	if (verbosity > -1) {
 		tracker_config_set_verbosity (config, verbosity);
 	}
 
-	tracker_dbus_init ();
+	if (!tracker_dbus_init ()) {
+		return EXIT_FAILURE;
+	}
 
 	/* Initialize subsystems */
 	initialize_directories ();
 
-	tracker_log_init (log_filename, tracker_config_get_verbosity (config));
+	tracker_log_init (tracker_config_get_verbosity (config),
+			  &log_filename);
 	g_print ("Starting log:\n  File:'%s'\n", log_filename);
-
-	tracker_thumbnailer_init (config, 0);
+	g_free (log_filename);
 
 	/* Make Tracker available for introspection */
 	if (!tracker_dbus_register_objects ()) {
-		g_free (log_filename);
 		g_object_unref (config);
 
 		return EXIT_FAILURE;
 	}
 
+	tracker_albumart_init ();
+
 	/* Main loop */
 	main_loop = g_main_loop_new (NULL, FALSE);
 	tracker_main_quit_timeout_reset ();
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
 
+	g_message ("Shutdown started");
+
 	/* Shutdown subsystems */
+	tracker_albumart_shutdown ();
 	tracker_dbus_shutdown ();
-	tracker_thumbnailer_shutdown ();
 	tracker_log_shutdown ();
 
-	g_free (log_filename);
 	g_object_unref (config);
 
+done:
+
+	if (log_handler_id != 0) {
+		/* Unset log handler */
+		g_log_remove_handler (NULL, log_handler_id);
+	}
+
 	return EXIT_SUCCESS;
 }
